#' #' @keywords internal
#' I <- function(d) {
#'   return(diag(d))
#' }
#' 
#' #' @keywords internal
#' J <- function(d) {
#'   return(rep(1,d)%*%t(rep(1,d)))
#' }
#' 
#' 
#' #' Hettmansperger-Norton Trend Test for k-Samples
#' #'
#' #' @description This function calculates the Hettmansperger-Norton trend test using pseudo-ranks under the null hypothesis H0F: F_1 = ... F_k = 0.
#' #' @param data numeric vector containing the data
#' #' @param group ordered factor vector for the groups
#' #' @param alternative either decreasing or increasing
#' #' @param formula formula object
#' #' @param trend custom numeric vector indicating the trend for the custom alternative, only used if alternative = "custom"
#' #' @param ... further arguments are ignored
#' #' @return Returns a data.frame with the results
#' #' @example R/example_2.txt
#' #' @references Brunner, E., Bathke A. C. and Konietschke, F. Rank- and Pseudo-Rank Procedures in Factorial Designs - Using R and SAS. Springer Verlag. to appear.
#' #' @references Hettmansperger, T. P., & Norton, R. M. (1987). Tests for patterned alternatives in k-sample problems. Journal of the American Statistical Association, 82(397), 292-299
#' #' @keywords internal
#' hettmansperger_norton_test_internal <- function(data, group, alternative = c("decreasing", "increasing", "custom"), formula = NULL, trend = NULL, ...) {
#'   
#'   stopifnot(is.numeric(data), is.factor(group), is.ordered(group))
#'   
#'   n <- as.numeric(as.matrix(table(group)))
#'   a <- length(n)
#'   df <- data.frame(pranks = psrank.numeric(data, group), group = group)
#'   df <- df[order(df$group),]
#'   pHat <- 1/sum(n)*(summaryBy(pranks~group,data=df, FUN = mean)[, 2]-1/2)
#'   n <- summaryBy(pranks~group,data=df, FUN = length)[, 2]
#'   alternative <- match.arg(alternative)
#'   w <- rep(1, a)
#'   switch(alternative, 
#'          decreasing={
#'            w <- a:1
#'          },
#'          increasing={
#'            w <- 1:a
#'          },
#'          custom={
#'            stopifnot(is.numeric(trend), length(trend)==a)
#'            w <- trend
#'          }
#'   )
#'   
#'   W <- diag(n)%*%(I(a) - 1/sum(n)*J(a)%*%diag(n))
#'   v2 <- 1/sum(n)^2*1/(sum(n)-1)*sum( (df$pranks - (sum(n)+1)/2 )^2 )
#'   sigmaHat2 <- sum(n)*v2*t(w)%*%W%*%diag(1/n)%*%W%*%w
#'   
#'   test <- sqrt(sum(n))*t(w)%*%W%*%pHat*1/sqrt(sigmaHat2)
#'   
#'   pValue <- 1 - pnorm(test)
#'   
#'   output <- list()
#'   output$test <- test
#'   output$pValue <- pValue
#'   output$ss <- n
#'   output$pHat <- pHat
#'   output$alternative <- alternative
#'   output$formula <- formula
#'   output$trend <- trend
#'   class(output) <- "pseudorank"
#'   
#'   return(output)
#'   
#' }
#' 
#' #' @keywords export
#' print.pseudorank <- function(x, ...) {
#'   cat("Hettmansperger-Norton Trend Test")
#'   cat("\n","\n")
#'   if(!is.null(x$formula)) {
#'     cat("Call:", "\n")
#'     print(x$formula)
#'     cat("\n")
#'   }
#'   cat("Alternative: ", x$alternative, "\n")
#'   if(x$alternative == "custom") {
#'     cat("Trend: ", x$trend, "\n")
#'   }
#'   cat("Test Statistic: ", x$test, "\n")
#'   cat("p-Value: ", x$pValue, "\n")
#'   cat("\n")
#'   cat("Descriptive:\n")
#'   df <- data.frame(n = x$ss, p = x$pHat)
#'   print(df, row.names = FALSE)
#' }
#' 
#' #' @keywords export
#' summary.pseudorank <- function(object, ...) {
#'   cat("Hettmansperger-Norton Trend Test")
#'   cat("\n", "\n")
#'   if(!is.null(object$formula)) {
#'     cat("Call:", "\n")
#'     print(object$formula)
#'     cat("\n")
#'   }
#'   cat("Alternative: ", object$alternative, "\n")
#'   if(object$alternative == "custom") {
#'     cat("Trend: ", object$trend, "\n")
#'   }
#'   cat("Test Statistic: ", object$test, "\n")
#'   cat("p-Value: ", object$pValue, "\n")
#'   cat("\n")
#'   cat("Descriptive:\n")
#'   df <- data.frame(n = object$ss, p = object$pHat)
#'   print(df, row.names = FALSE)
#' }
